@@ -22,6 +22,7 @@
 #include <cplusplus/AST.h>
 #include <cplusplus/ASTPath.h>
 #include <cplusplus/Icons.h>
+#include <cppeditor/compilationdb.h>
 #include <cppeditor/cppcodemodelsettings.h>
 #include <cppeditor/cppeditorconstants.h>
 #include <cppeditor/cppeditorwidget.h>
@@ -44,8 +45,9 @@
 #include <languageserverprotocol/progresssupport.h>
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/devicesupport/idevice.h>
-#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/kitaspects.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/target.h>
@@ -53,12 +55,13 @@
 #include <texteditor/codeassist/assistinterface.h>
 #include <texteditor/codeassist/iassistprocessor.h>
 #include <texteditor/codeassist/iassistprovider.h>
-#include <texteditor/codeassist/textdocumentmanipulatorinterface.h>
 #include <texteditor/texteditor.h>
 #include <utils/algorithm.h>
 #include <utils/async.h>
+#include <utils/clangutils.h>
 #include <utils/environment.h>
 #include <utils/fileutils.h>
+#include <utils/mimeconstants.h>
 #include <utils/itemviews.h>
 #include <utils/theme/theme.h>
 #include <utils/utilsicons.h>
@@ -72,9 +75,7 @@
 #include <QRegularExpression>
 
 #include <cmath>
-#include <new>
 #include <optional>
-#include <set>
 #include <unordered_map>
 #include <utility>
 
@@ -89,6 +90,8 @@ using namespace Utils;
 namespace ClangCodeModel {
 namespace Internal {
 
+using Key = LanguageServerProtocol::Key;
+
 Q_LOGGING_CATEGORY(clangdLog, "qtc.clangcodemodel.clangd", QtWarningMsg);
 Q_LOGGING_CATEGORY(clangdLogAst, "qtc.clangcodemodel.clangd.ast", QtWarningMsg);
 static Q_LOGGING_CATEGORY(clangdLogServer, "qtc.clangcodemodel.clangd.server", QtWarningMsg);
@@ -99,7 +102,7 @@ class SymbolDetails : public JsonObject
 public:
     using JsonObject::JsonObject;
 
-    static constexpr char16_t usrKey[] = u"usr";
+    static constexpr Key usrKey{"usr"};
 
     // the unqualified name of the symbol
     QString name() const { return typedValue<QString>(nameKey); }
@@ -141,7 +144,7 @@ private:
         case Qt::ForegroundRole:
             if ((detail().endsWith("class") || detail().endsWith("struct"))
                 && range().end() == selectionRange().end()) {
-                return creatorTheme()->color(Theme::TextColorDisabled);
+                return creatorColor(Theme::TextColorDisabled);
             }
             break;
         }
@@ -206,17 +209,20 @@ static BaseClientInterface *clientInterface(Project *project, const Utils::FileP
                             "--clang-tidy=0"}};
     if (settings.workerThreadLimit() != 0)
         cmd.addArg("-j=" + QString::number(settings.workerThreadLimit()));
-    if (indexingEnabled && settings.clangdVersion() >= QVersionNumber(15)) {
+    if (indexingEnabled) {
         cmd.addArg("--background-index-priority="
                    + ClangdSettings::priorityToString(indexingPriority));
     }
-    if (settings.clangdVersion() >= QVersionNumber(16))
-        cmd.addArg("--rename-file-limit=0");
+    cmd.addArg("--rename-file-limit=0");
     if (!jsonDbDir.isEmpty())
         cmd.addArg("--compile-commands-dir=" + clangdExePath.withNewMappedPath(jsonDbDir).path());
     if (clangdLogServer().isDebugEnabled())
-        cmd.addArgs({"--log=verbose", "--pretty"});
+        cmd.addArgs({"--log=verbose", "--pretty", "--hidden-features=1"});
     cmd.addArg("--use-dirty-headers");
+    if (settings.completionRankingModel() != ClangdSettings::CompletionRankingModel::Default) {
+        cmd.addArg("--ranking-model=" + ClangdSettings::rankingModelToCmdLineString(
+                       settings.completionRankingModel()));
+    }
     const auto interface = new StdIOClientInterface;
     interface->setCommandLine(cmd);
     return interface;
@@ -226,15 +232,15 @@ class DiagnosticsCapabilities : public JsonObject
 {
 public:
     using JsonObject::JsonObject;
-    void enableCategorySupport() { insert(u"categorySupport", true); }
-    void enableCodeActionsInline() {insert(u"codeActionsInline", true);}
+    void enableCategorySupport() { insert(Key{"categorySupport"}, true); }
+    void enableCodeActionsInline() {insert(Key{"codeActionsInline"}, true);}
 };
 
 class InactiveRegionsCapabilities : public JsonObject
 {
 public:
     using JsonObject::JsonObject;
-    void enableInactiveRegionsSupport() { insert(u"inactiveRegions", true); }
+    void enableInactiveRegionsSupport() { insert(Key{"inactiveRegions"}, true); }
 };
 
 class ClangdTextDocumentClientCapabilities : public TextDocumentClientCapabilities
@@ -243,9 +249,9 @@ public:
     using TextDocumentClientCapabilities::TextDocumentClientCapabilities;
 
     void setPublishDiagnostics(const DiagnosticsCapabilities &caps)
-    { insert(u"publishDiagnostics", caps); }
+    { insert(Key{"publishDiagnostics"}, caps); }
     void setInactiveRegionsCapabilities(const InactiveRegionsCapabilities &caps)
-    { insert(u"inactiveRegionsCapabilities", caps); }
+    { insert(Key{"inactiveRegionsCapabilities"}, caps); }
 };
 
 static qint64 getRevision(const TextDocument *doc)
@@ -343,14 +349,13 @@ public:
 
     ClangdClient * const q;
     const CppEditor::ClangdSettings::Data settings;
-    ClangdFollowSymbol *followSymbol = nullptr;
+    QList<ClangdFollowSymbol *> followSymbolOps;
     ClangdSwitchDeclDef *switchDeclDef = nullptr;
     ClangdFindLocalReferences *findLocalRefs = nullptr;
     std::optional<QVersionNumber> versionNumber;
 
     QHash<TextDocument *, HighlightingData> highlightingData;
     QHash<Utils::FilePath, CppEditor::BaseEditorDocumentParser::Configuration> parserConfigs;
-    QHash<Utils::FilePath, Tasks> issuePaneEntries;
     QHash<Utils::FilePath, int> openedExtraFiles;
 
     VersionedDataCache<const TextDocument *, ClangdAstNode> astCache;
@@ -378,7 +383,7 @@ static void addToCompilationDb(QJsonObject &cdb,
     const QString fileString = sourceFile.path.toUserOutput();
     args.append(fileString);
     QJsonObject value;
-    value.insert("workingDirectory", workingDir.toString());
+    value.insert("workingDirectory", workingDir.path());
     value.insert("compilationCommand", args);
     cdb.insert(fileString, value);
 }
@@ -393,23 +398,28 @@ ClangdClient::ClangdClient(Project *project, const Utils::FilePath &jsonDbDir, c
 {
     setName(Tr::tr("clangd"));
     LanguageFilter langFilter;
-    using namespace CppEditor::Constants;
+    using namespace Utils::Constants;
     langFilter.mimeTypes = QStringList{C_HEADER_MIMETYPE, C_SOURCE_MIMETYPE,
             CPP_HEADER_MIMETYPE, CPP_SOURCE_MIMETYPE, OBJECTIVE_CPP_SOURCE_MIMETYPE,
             OBJECTIVE_C_SOURCE_MIMETYPE, CUDA_SOURCE_MIMETYPE};
     setSupportedLanguage(langFilter);
     setActivateDocumentAutomatically(true);
     setCompletionAssistProvider(new ClangdCompletionAssistProvider(this));
+    setFunctionHintAssistProvider(new ClangdFunctionHintProvider(this));
     setQuickFixAssistProvider(new ClangdQuickFixProvider(this));
     symbolSupport().setLimitRenamingToProjects(true);
+    symbolSupport().setRenameResultsEnhancer([](const SearchResultItems &symbolOccurrencesInCode) {
+        return CppEditor::symbolOccurrencesInDeclarationComments(symbolOccurrencesInCode);
+    });
     if (!project) {
         QJsonObject initOptions;
         const Utils::FilePath includeDir
                 = CppEditor::ClangdSettings(d->settings).clangdIncludePath();
         CppEditor::CompilerOptionsBuilder optionsBuilder = clangOptionsBuilder(
-                    *CppEditor::CppModelManager::instance()->fallbackProjectPart(),
+                    *CppEditor::CppModelManager::fallbackProjectPart(),
                     warningsConfigForProject(nullptr), includeDir, {});
-        const CppEditor::UsePrecompiledHeaders usePch = CppEditor::getPchUsage();
+        const CppEditor::UsePrecompiledHeaders usePch
+            = CppEditor::CppCodeModelSettings::usePrecompiledHeaders(nullptr);
         const QJsonArray projectPartOptions = fullProjectPartOptions(
                     optionsBuilder, globalClangOptions());
         const QJsonArray clangOptions = clangOptionsForFile({}, optionsBuilder.projectPart(),
@@ -491,8 +501,8 @@ ClangdClient::ClangdClient(Project *project, const Utils::FilePath &jsonDbDir, c
 
 ClangdClient::~ClangdClient()
 {
-    if (d->followSymbol)
-        d->followSymbol->clear();
+    for (ClangdFollowSymbol * const followSymbol : std::as_const(d->followSymbolOps))
+        followSymbol->clear();
     delete d;
 }
 
@@ -510,13 +520,20 @@ void ClangdClient::openExtraFile(const Utils::FilePath &filePath, const QString
         return;
     }
 
-    QFile cxxFile(filePath.toString());
-    if (content.isEmpty() && !cxxFile.open(QIODevice::ReadOnly))
-        return;
+    QString text;
+    if (!content.isEmpty()) {
+        text = content;
+    } else {
+        expected_str<QByteArray> fileContent = filePath.fileContents();
+        if (!fileContent)
+            return;
+        text = QString::fromUtf8(*std::move(fileContent));
+    }
+
     TextDocumentItem item;
     item.setLanguageId("cpp");
     item.setUri(hostPathToServerUri(filePath));
-    item.setText(!content.isEmpty() ? content : QString::fromUtf8(cxxFile.readAll()));
+    item.setText(std::move(text));
     item.setVersion(0);
     sendMessage(DidOpenTextDocumentNotification(DidOpenTextDocumentParams(item)),
                 SendDocUpdates::Ignore);
@@ -547,9 +564,7 @@ void ClangdClient::findUsages(const CppEditor::CursorInEditor &cursor,
     if (searchTerm.isEmpty())
         return;
 
-    if (replacement && versionNumber() >= QVersionNumber(16)
-        && Utils::qtcEnvironmentVariable("QTC_CLANGD_RENAMING") != "0") {
-
+    if (replacement && Utils::qtcEnvironmentVariable("QTC_CLANGD_RENAMING") != "0") {
         // If we have up-to-date highlighting data, we can prevent giving clangd
         // macros or namespaces to rename, which it can't cope with.
         // TODO: Fix this upstream for macros; see https://github.com/clangd/clangd/issues/729.
@@ -584,12 +599,13 @@ void ClangdClient::findUsages(const CppEditor::CursorInEditor &cursor,
 
         if (useClangdForRenaming) {
             symbolSupport().renameSymbol(cursor.textDocument(), adjustedCursor, *replacement,
-                                         renameCallback, CppEditor::preferLowerCaseFileNames());
+                                         renameCallback,
+                                         CppEditor::preferLowerCaseFileNames(project()));
             return;
         }
     }
 
-    const bool categorize = CppEditor::codeModelSettings()->categorizeFindReferences();
+    const bool categorize = CppEditor::CppCodeModelSettings::categorizeFindReferences();
 
     // If it's a "normal" symbol, go right ahead.
     if (searchTerm != "operator" && Utils::allOf(searchTerm, [](const QChar &c) {
@@ -678,30 +694,14 @@ const LanguageClient::Client::CustomInspectorTabs ClangdClient::createCustomInsp
 
 class ClangdDiagnosticManager : public LanguageClient::DiagnosticManager
 {
-    using LanguageClient::DiagnosticManager::DiagnosticManager;
-
-    ClangdClient *getClient() const { return qobject_cast<ClangdClient *>(client()); }
-
-    bool isCurrentDocument(const Utils::FilePath &filePath) const
+public:
+    ClangdDiagnosticManager(LanguageClient::Client *client)
+        : LanguageClient::DiagnosticManager(client)
     {
-        const IDocument * const doc = EditorManager::currentDocument();
-        return doc && doc->filePath() == filePath;
-    }
-
-    void showDiagnostics(const Utils::FilePath &filePath, int version) override
-    {
-        getClient()->clearTasks(filePath);
-        DiagnosticManager::showDiagnostics(filePath, version);
-        if (isCurrentDocument(filePath))
-            getClient()->switchIssuePaneEntries(filePath);
-    }
-
-    void hideDiagnostics(const Utils::FilePath &filePath) override
-    {
-        DiagnosticManager::hideDiagnostics(filePath);
-        if (isCurrentDocument(filePath))
-            TaskHub::clearTasks(Constants::TASK_CATEGORY_DIAGNOSTICS);
+        setTaskCategory(Constants::TASK_CATEGORY_DIAGNOSTICS);
+        setForceCreateTasks(false);
     }
+private:
 
     QList<Diagnostic> filteredDiagnostics(const QList<Diagnostic> &diagnostics) const override
     {
@@ -717,7 +717,18 @@ class ClangdDiagnosticManager : public LanguageClient::DiagnosticManager
                              const Diagnostic &diagnostic,
                              bool isProjectFile) const override
     {
-        return new ClangdTextMark(doc, diagnostic, isProjectFile, getClient());
+        return new ClangdTextMark(
+                    doc, diagnostic, isProjectFile, qobject_cast<ClangdClient *>(client()));
+    }
+
+    QString taskText(const Diagnostic &diagnostic) const override
+    {
+        QString text = diagnostic.message();
+        auto splitIndex = text.indexOf("\n\n");
+        if (splitIndex >= 0)
+            text.truncate(splitIndex);
+
+        return diagnosticCategoryPrefixRemoved(text);
     }
 };
 
@@ -754,10 +765,25 @@ bool ClangdClient::fileBelongsToProject(const Utils::FilePath &filePath) const
     return Client::fileBelongsToProject(filePath);
 }
 
-RefactoringChangesData *ClangdClient::createRefactoringChangesBackend() const
+QList<Text::Range> ClangdClient::additionalDocumentHighlights(
+    TextEditorWidget *editorWidget, const QTextCursor &cursor)
 {
-    return new CppEditor::CppRefactoringChangesData(
-                CppEditor::CppModelManager::instance()->snapshot());
+    return CppEditor::symbolOccurrencesInDeclarationComments(
+        qobject_cast<CppEditor::CppEditorWidget *>(editorWidget), cursor);
+}
+
+bool ClangdClient::shouldSendDidSave(const TextEditor::TextDocument *doc) const
+{
+    for (const Project * const p : ProjectManager::projects()) {
+        if (const Node * const n  = p->nodeForFilePath(doc->filePath()))
+            return n->asFileNode() && n->asFileNode()->fileType() == FileType::Header;
+    }
+    return CppEditor::ProjectFile::isHeader(doc->filePath());
+}
+
+RefactoringFilePtr ClangdClient::createRefactoringFile(const FilePath &filePath) const
+{
+    return CppEditor::CppRefactoringChanges(CppEditor::CppModelManager::snapshot()).file(filePath);
 }
 
 QVersionNumber ClangdClient::versionNumber() const
@@ -877,8 +903,7 @@ void ClangdClient::updateParserConfig(const Utils::FilePath &filePath,
     // TODO: Should we write the editor defines into the json file? It seems strange
     //       that they should affect the index only while the file is open in the editor.
     const auto projectPart = !config.preferredProjectPartId.isEmpty()
-            ? CppEditor::CppModelManager::instance()->projectPartForId(
-                  config.preferredProjectPartId)
+            ? CppEditor::CppModelManager::projectPartForId(config.preferredProjectPartId)
             : projectPartForFile(filePath);
     if (!projectPart)
         return;
@@ -900,12 +925,18 @@ void ClangdClient::updateParserConfig(const Utils::FilePath &filePath,
     CppEditor::CompilerOptionsBuilder optionsBuilder = clangOptionsBuilder(
                 *projectPart, warningsConfigForProject(project()), includeDir,
                 ProjectExplorer::Macro::toMacros(config.editorDefines));
-    const CppEditor::ProjectFile file(filePath,
-                                      CppEditor::ProjectFile::classify(filePath.toString()));
+    const CppEditor::ProjectFile file(filePath, CppEditor::ProjectFile::classify(filePath));
     const QJsonArray projectPartOptions = fullProjectPartOptions(
                 optionsBuilder, globalClangOptions());
-    addToCompilationDb(cdbChanges, *projectPart, CppEditor::getPchUsage(), projectPartOptions,
-                       filePath.parentDir(), file, optionsBuilder.isClStyle());
+    const auto cppSettings = CppEditor::CppCodeModelSettings::settingsForProject(
+        projectPart->topLevelProject);
+    addToCompilationDb(cdbChanges,
+                       *projectPart,
+                       cppSettings.usePrecompiledHeaders(),
+                       projectPartOptions,
+                       filePath.parentDir(),
+                       file,
+                       optionsBuilder.isClStyle());
     QJsonObject settings;
     addCompilationDb(settings, cdbChanges);
     DidChangeConfigurationParams configChangeParams;
@@ -914,24 +945,6 @@ void ClangdClient::updateParserConfig(const Utils::FilePath &filePath,
     emit configChanged();
 }
 
-void ClangdClient::switchIssuePaneEntries(const FilePath &filePath)
-{
-    TaskHub::clearTasks(Constants::TASK_CATEGORY_DIAGNOSTICS);
-    const Tasks tasks = d->issuePaneEntries.value(filePath);
-    for (const Task &t : tasks)
-        TaskHub::addTask(t);
-}
-
-void ClangdClient::addTask(const ProjectExplorer::Task &task)
-{
-    d->issuePaneEntries[task.file] << task;
-}
-
-void ClangdClient::clearTasks(const Utils::FilePath &filePath)
-{
-    d->issuePaneEntries[filePath].clear();
-}
-
 std::optional<bool> ClangdClient::hasVirtualFunctionAt(TextDocument *doc, int revision,
                                                          const Range &range)
 {
@@ -983,7 +996,7 @@ MessageId ClangdClient::requestSymbolInfo(const Utils::FilePath &filePath, const
 #ifdef WITH_TESTS
 ClangdFollowSymbol *ClangdClient::currentFollowSymbolOperation()
 {
-    return d->followSymbol;
+    return d->followSymbolOps.isEmpty() ? nullptr : d->followSymbolOps.first();
 }
 #endif
 
@@ -998,25 +1011,41 @@ void ClangdClient::followSymbol(TextDocument *document,
 {
     QTC_ASSERT(documentOpen(document), openDocument(document));
 
-    if (d->followSymbol)
-        d->followSymbol->cancel();
+    const ClangdFollowSymbol::Origin origin
+        = CppEditor::CppCodeModelSettings::isInteractiveFollowSymbol()
+              ? ClangdFollowSymbol::Origin::User
+              : ClangdFollowSymbol::Origin::Code;
+    if (origin == ClangdFollowSymbol::Origin::User) {
+        for (auto it = d->followSymbolOps.begin(); it != d->followSymbolOps.end(); ) {
+            ClangdFollowSymbol * const followSymbol = *it;
+            if (followSymbol->isInteractive()) {
+                it = d->followSymbolOps.erase(it);
+                followSymbol->cancel();
+            } else {
+                ++it;
+            }
+        }
+    }
 
     const QTextCursor adjustedCursor = d->adjustedCursor(cursor, document);
     if (followTo == FollowTo::SymbolDef && !resolveTarget) {
-        symbolSupport().findLinkAt(document, adjustedCursor, callback, false);
+        symbolSupport().findLinkAt(document,
+                                   adjustedCursor,
+                                   callback,
+                                   false,
+                                   LanguageClient::LinkTarget::SymbolDef);
         return;
     }
 
     qCDebug(clangdLog) << "follow symbol requested" << document->filePath()
                        << adjustedCursor.blockNumber() << adjustedCursor.positionInBlock();
-    auto clangdFollowSymbol = new ClangdFollowSymbol(this, adjustedCursor, editorWidget, document,
-                                                     callback, followTo, openInSplit);
+    auto clangdFollowSymbol = new ClangdFollowSymbol(this, origin, adjustedCursor, editorWidget,
+                                                     document, callback, followTo, openInSplit);
     connect(clangdFollowSymbol, &ClangdFollowSymbol::done, this, [this, clangdFollowSymbol] {
         clangdFollowSymbol->deleteLater();
-        if (clangdFollowSymbol == d->followSymbol)
-            d->followSymbol = nullptr;
+        d->followSymbolOps.removeOne(clangdFollowSymbol);
     });
-    d->followSymbol = clangdFollowSymbol;
+    d->followSymbolOps << clangdFollowSymbol;
 }
 
 void ClangdClient::switchDeclDef(TextDocument *document, const QTextCursor &cursor,
@@ -1059,9 +1088,11 @@ void ClangdClient::switchHeaderSource(const Utils::FilePath &filePath, bool inNe
     sendMessage(req);
 }
 
-void ClangdClient::findLocalUsages(TextDocument *document, const QTextCursor &cursor,
-        CppEditor::RenameCallback &&callback)
+void ClangdClient::findLocalUsages(CppEditor::CppEditorWidget *editorWidget,
+                                   const QTextCursor &cursor, CppEditor::RenameCallback &&callback)
 {
+    QTC_ASSERT(editorWidget, return);
+    TextDocument * const document = editorWidget->textDocument();
     QTC_ASSERT(documentOpen(document), openDocument(document));
 
     qCDebug(clangdLog) << "local references requested" << document->filePath()
@@ -1079,7 +1110,7 @@ void ClangdClient::findLocalUsages(TextDocument *document, const QTextCursor &cu
         return;
     }
 
-    d->findLocalRefs = new ClangdFindLocalReferences(this, document, cursor, callback);
+    d->findLocalRefs = new ClangdFindLocalReferences(this, editorWidget, cursor, callback);
     connect(d->findLocalRefs, &ClangdFindLocalReferences::done, this, [this] {
         d->findLocalRefs->deleteLater();
         d->findLocalRefs = nullptr;
@@ -1119,14 +1150,15 @@ void ClangdClient::gatherHelpItemForTooltip(const HoverRequest::Response &hoverR
                 for (const QString &line : lines) {
                     const QString possibleFilePath = line.simplified();
                     const auto looksLikeFilePath = [&] {
-                        if (possibleFilePath.length() < 3)
+                        if (possibleFilePath.length() < 4)
                             return false;
                         if (osType() == OsTypeWindows) {
-                            if (possibleFilePath.startsWith(R"(\\)"))
+                            if (possibleFilePath.startsWith(R"(\\\\)"))
                                 return true;
                             return possibleFilePath.front().isLetter()
                                     && possibleFilePath.at(1) == ':'
-                                    && possibleFilePath.at(2) == '\\';
+                                    && possibleFilePath.at(2) == '\\'
+                                    && possibleFilePath.at(3) == '\\';
                         }
                         return possibleFilePath.front() == '/'
                                 && possibleFilePath.at(1).isLetterOrNumber();
@@ -1475,10 +1507,9 @@ void ClangdClient::Private::setHelpItemForTooltip(const MessageId &token,
         mark = type;
 
     const HelpItem helpItem(helpIds, filePath, mark, category);
+    q->hoverHandler()->setHelpItem(token, helpItem);
     if (isTesting)
-        emit q->helpItemGathered(helpItem);
-    else
-        q->hoverHandler()->setHelpItem(token, helpItem);
+        emit q->helpItemGathered(helpItem, q->hoverHandler()->toolTip());
 }
 
 // Unfortunately, clangd ignores almost everything except symbols when sending
@@ -1506,7 +1537,7 @@ void ClangdClient::Private::handleSemanticTokens(TextDocument *doc,
         return;
     }
     force = force || isTesting;
-    const auto data = highlightingData.find(doc);
+    auto data = highlightingData.find(doc);
     if (data != highlightingData.end()) {
         if (!force && data->previousTokens.first == tokens
                 && data->previousTokens.second == version) {
@@ -1516,66 +1547,55 @@ void ClangdClient::Private::handleSemanticTokens(TextDocument *doc,
         data->previousTokens.first = tokens;
         data->previousTokens.second = version;
     } else {
-        highlightingData.insert(doc, {{tokens, version}, {}});
+        data = highlightingData.insert(doc, {{tokens, version}, {}});
     }
     for (const ExpandedSemanticToken &t : tokens)
         qCDebug(clangdLogHighlight()) << '\t' << t.line << t.column << t.length << t.type
                                       << t.modifiers;
 
-    const auto astHandler = [this, tokens, doc, version](const ClangdAstNode &ast, const MessageId &) {
-        FinalizingSubtaskTimer t(highlightingTimer);
-        if (!q->documentOpen(doc))
-            return;
-        if (version != q->documentVersion(doc->filePath())) {
-            qCInfo(clangdLogHighlight) << "AST not up to date; aborting highlighting procedure"
-                                        << version << q->documentVersion(doc->filePath());
-            return;
+    FinalizingSubtaskTimer ft(highlightingTimer);
+    if (!q->documentOpen(doc))
+        return;
+    if (version != q->documentVersion(doc->filePath())) {
+        qCInfo(clangdLogHighlight) << "AST not up to date; aborting highlighting procedure"
+                                   << version << q->documentVersion(doc->filePath());
+        return;
+    }
+
+    const auto runner = [tokens, filePath = doc->filePath(),
+                         text = doc->document()->toPlainText(),
+                         rev = doc->document()->revision(), this] {
+        try {
+            return Utils::asyncRun(doSemanticHighlighting, filePath, tokens, text,
+                                   rev, highlightingTimer);
+        } catch (const std::exception &e) {
+            qWarning() << "caught" << e.what() << "in main highlighting thread";
+            return QFuture<HighlightingResult>();
         }
-        if (clangdLogAst().isDebugEnabled())
-            ast.print();
-
-        const auto runner = [tokens, filePath = doc->filePath(),
-                             text = doc->document()->toPlainText(), ast,
-                             doc = QPointer(doc), rev = doc->document()->revision(),
-                             clangdVersion = q->versionNumber(),
-                             this] {
-            try {
-                return Utils::asyncRun(doSemanticHighlighting, filePath, tokens, text, ast, doc,
-                                       rev, clangdVersion, highlightingTimer);
-            } catch (const std::exception &e) {
-                qWarning() << "caught" << e.what() << "in main highlighting thread";
-                return QFuture<HighlightingResult>();
-            }
-        };
-
-        if (isTesting) {
-            const auto watcher = new QFutureWatcher<HighlightingResult>(q);
-            connect(watcher, &QFutureWatcher<HighlightingResult>::finished,
-                    q, [this, watcher, fp = doc->filePath()] {
-                emit q->highlightingResultsReady(watcher->future().results(), fp);
-                watcher->deleteLater();
-            });
-            watcher->setFuture(runner());
-            return;
-        }
-
-        auto &data = highlightingData[doc];
-        if (!data.highlighter)
-            data.highlighter = new CppEditor::SemanticHighlighter(doc);
-        else
-            data.highlighter->updateFormatMapFromFontSettings();
-        data.highlighter->setHighlightingRunner(runner);
-        data.highlighter->run();
     };
-    if (q->versionNumber().majorVersion() >= 17)
-        astHandler({}, {});
+
+    if (isTesting) {
+        const auto watcher = new QFutureWatcher<HighlightingResult>(q);
+        connect(watcher, &QFutureWatcher<HighlightingResult>::finished,
+                q, [this, watcher, fp = doc->filePath()] {
+            emit q->highlightingResultsReady(watcher->future().results(), fp);
+            watcher->deleteLater();
+        });
+        watcher->setFuture(runner());
+        return;
+    }
+
+    if (!data->highlighter)
+        data->highlighter = new CppEditor::SemanticHighlighter(doc);
     else
-        getAndHandleAst(doc, astHandler, AstCallbackMode::SyncIfPossible);
+        data->highlighter->updateFormatMapFromFontSettings();
+    data->highlighter->setHighlightingRunner(runner);
+    data->highlighter->run();
 }
 
 std::optional<QList<CodeAction> > ClangdDiagnostic::codeActions() const
 {
-    auto actions = optionalArray<LanguageServerProtocol::CodeAction>(u"codeActions");
+    auto actions = optionalArray<LanguageServerProtocol::CodeAction>(Key{"codeActions"});
     if (!actions)
         return actions;
     static const QStringList badCodeActions{
@@ -1592,7 +1612,7 @@ std::optional<QList<CodeAction> > ClangdDiagnostic::codeActions() const
 
 QString ClangdDiagnostic::category() const
 {
-    return typedValue<QString>(u"category");
+    return typedValue<QString>(Key{"category"});
 }
 
 MessageId ClangdClient::Private::getAndHandleAst(const TextDocOrFile &doc,
