@@ -72,7 +72,7 @@
 */
 
 /*!
-    \enum IDocument::OpenResult
+    \enum Core::IDocument::OpenResult
 
     The OpenResult enum describes whether a file was successfully opened.
 
@@ -87,13 +87,13 @@
 */
 
 /*!
-    \enum IDocument::ReloadSetting
+    \enum Core::IDocument::ReloadSetting
 
     \internal
 */
 
 /*!
-    \enum IDocument::ChangeTrigger
+    \enum Core::IDocument::ChangeTrigger
 
     The ChangeTrigger enum describes whether a file was changed from \QC
     internally or from the outside.
@@ -107,7 +107,7 @@
 */
 
 /*!
-    \enum IDocument::ChangeType
+    \enum Core::IDocument::ChangeType
 
     The ChangeType enum describes the way in which the file changed.
 
@@ -121,7 +121,7 @@
 */
 
 /*!
-    \enum IDocument::ReloadFlag
+    \enum Core::IDocument::ReloadFlag
 
     The ReloadFlag enum describes if a file should be reloaded from disk.
 
@@ -326,6 +326,7 @@ IDocument::OpenResult IDocument::open(QString *errorString, const Utils::FilePat
 
 /*!
     Saves the contents of the document to the \a filePath on disk.
+    If \a filePath is empty filePath() is used.
 
     If \a autoSave is \c true, the saving is done for an auto-save, so the
     document should avoid cleanups or other operations that it does for
@@ -340,14 +341,16 @@ IDocument::OpenResult IDocument::open(QString *errorString, const Utils::FilePat
     \sa shouldAutoSave()
     \sa aboutToSave()
     \sa saved()
+    \sa filePath()
 */
-bool IDocument::save(QString *errorString, const Utils::FilePath &filePath, bool autoSave)
+Result IDocument::save(const FilePath &filePath, bool autoSave)
 {
-    emit aboutToSave(filePath.isEmpty() ? this->filePath() : filePath, autoSave);
-    const bool success = saveImpl(errorString, filePath, autoSave);
-    if (success)
-        emit saved(filePath.isEmpty() ? this->filePath() : filePath, autoSave);
-    return success;
+    const FilePath savePath = filePath.isEmpty() ? this->filePath() : filePath;
+    emit aboutToSave(savePath, autoSave);
+    const Result res = saveImpl(savePath, autoSave);
+    if (res)
+        emit saved(savePath, autoSave);
+    return res;
 }
 
 /*!
@@ -357,18 +360,15 @@ bool IDocument::save(QString *errorString, const Utils::FilePath &filePath, bool
     document should avoid cleanups or other operations that it does for
     user-requested saves.
 
-    Use \a errorString to return an error message if saving failed.
-
-    Returns whether saving was successful.
+    Returns whether saving was successful, including an error message when it was not.
 
     The default implementation does nothing and returns \c false.
 */
-bool IDocument::saveImpl(QString *errorString, const Utils::FilePath &filePath, bool autoSave)
+Result IDocument::saveImpl(const FilePath &filePath, bool autoSave)
 {
-    Q_UNUSED(errorString)
     Q_UNUSED(filePath)
     Q_UNUSED(autoSave)
-    return false;
+    return Result::Error(Tr::tr("Not implemented"));
 }
 
 /*!
@@ -401,6 +401,13 @@ bool IDocument::setContents(const QByteArray &contents)
     return false;
 }
 
+/*!
+    Formats the contents of the document, if the implementation supports such functionality.
+*/
+void IDocument::formatContents()
+{
+}
+
 /*!
     Returns the absolute path of the file that this document refers to. May be
     empty for documents that are not backed by a file.
@@ -460,12 +467,11 @@ IDocument::ReloadBehavior IDocument::reloadBehavior(ChangeTrigger trigger, Chang
     \sa reloadFinished()
     \sa changed()
 */
-bool IDocument::reload(QString *errorString, ReloadFlag flag, ChangeType type)
+Result IDocument::reload(ReloadFlag flag, ChangeType type)
 {
-    Q_UNUSED(errorString)
     Q_UNUSED(flag)
     Q_UNUSED(type)
-    return true;
+    return Result::Ok;
 }
 
 /*!
@@ -641,12 +647,13 @@ void IDocument::setMimeType(const QString &mimeType)
 /*!
     \internal
 */
-bool IDocument::autoSave(QString *errorString, const FilePath &filePath)
+Result IDocument::autoSave(const FilePath &filePath)
 {
-    if (!save(errorString, filePath, true))
-        return false;
+    if (const Result res = save(filePath, true); !res)
+        return res;
+
     d->autoSavePath = filePath;
-    return true;
+    return Result::Ok;
 }
 
 static const char kRestoredAutoSave[] = "RestoredAutoSave";
