@@ -1,55 +1,41 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "iosconfigurations.h"
+
 #include "iosconstants.h"
 #include "iosdevice.h"
-#include "iossimulator.h"
-#include "simulatorcontrol.h"
 #include "iosprobe.h"
+#include "iossimulator.h"
+#include "iostr.h"
+#include "simulatorcontrol.h"
 
 #include <coreplugin/icore.h>
-#include <utils/algorithm.h>
-#include <utils/synchronousprocess.h>
-#include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+
+#include <projectexplorer/kitaspects.h>
 #include <projectexplorer/kitmanager.h>
-#include <projectexplorer/kitinformation.h>
 #include <projectexplorer/devicesupport/devicemanager.h>
+#include <projectexplorer/devicesupport/idevice.h>
 #include <projectexplorer/toolchainmanager.h>
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/gcctoolchain.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/toolchainconfigwidget.h>
+
 #include <debugger/debuggeritemmanager.h>
 #include <debugger/debuggeritem.h>
-#include <debugger/debuggerkitinformation.h>
+#include <debugger/debuggerkitaspect.h>
+
 #include <qtsupport/baseqtversion.h>
-#include <qtsupport/qtkitinformation.h>
+#include <qtsupport/qtkitaspect.h>
 #include <qtsupport/qtversionmanager.h>
 #include <qtsupport/qtversionfactory.h>
 
+#include <utils/algorithm.h>
+#include <utils/futuresynchronizer.h>
+#include <utils/qtcassert.h>
+#include <utils/qtcprocess.h>
+
 #include <QDir>
 #include <QDomDocument>
 #include <QFileInfo>
@@ -57,10 +43,9 @@
 #include <QHash>
 #include <QList>
 #include <QLoggingCategory>
-#include <QProcess>
 #include <QSettings>
-#include <QStringList>
 #include <QStandardPaths>
+#include <QStringList>
 #include <QTimer>
 
 #include <memory>
@@ -71,17 +56,18 @@ using namespace Utils;
 using namespace Debugger;
 
 namespace {
-Q_LOGGING_CATEGORY(kitSetupLog, "qtc.ios.kitSetup", QtWarningMsg)
-Q_LOGGING_CATEGORY(iosCommonLog, "qtc.ios.common", QtWarningMsg)
+static Q_LOGGING_CATEGORY(kitSetupLog, "qtc.ios.kitSetup", QtWarningMsg)
+static Q_LOGGING_CATEGORY(iosCommonLog, "qtc.ios.common", QtWarningMsg)
 }
 
-using ToolChainPair = std::pair<ClangToolChain *, ClangToolChain *>;
+using ToolchainPair = std::pair<GccToolchain *, GccToolchain *>;
 namespace Ios {
 namespace Internal {
 
+const bool IgnoreAllDevicesDefault = false;
+
 const char SettingsGroup[] = "IosConfigurations";
 const char ignoreAllDevicesKey[] = "IgnoreAllDevices";
-const char screenshotDirPathKey[] = "ScreeshotDirPath";
 
 const char provisioningTeamsTag[] = "IDEProvisioningTeams";
 const char freeTeamTag[] = "isFreeProvisioningTeam";
@@ -98,7 +84,7 @@ const char profileTeamIdTag[] = "TeamIdentifier";
 static const QString xcodePlistPath = QDir::homePath() + "/Library/Preferences/com.apple.dt.Xcode.plist";
 static const QString provisioningProfileDirPath = QDir::homePath() + "/Library/MobileDevice/Provisioning Profiles";
 
-static Core::Id deviceId(const QString &sdkName)
+static Id deviceId(const QString &sdkName)
 {
     if (sdkName.startsWith("iphoneos", Qt::CaseInsensitive))
         return Constants::IOS_DEVICE_TYPE;
@@ -107,68 +93,68 @@ static Core::Id deviceId(const QString &sdkName)
     return {};
 }
 
-static bool isSimulatorDeviceId(const Core::Id &id)
+static bool isSimulatorDeviceId(const Id &id)
 {
     return id == Constants::IOS_SIMULATOR_TYPE;
 }
 
-static QList<ClangToolChain *> clangToolChains(const QList<ToolChain *> &toolChains)
+static QList<GccToolchain *> clangToolchains(const Toolchains &toolchains)
 {
-    QList<ClangToolChain *> clangToolChains;
-    foreach (ToolChain *toolChain, toolChains)
-        if (toolChain->typeId() == ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID)
-            clangToolChains.append(static_cast<ClangToolChain *>(toolChain));
-    return clangToolChains;
+    QList<GccToolchain *> clangToolchains;
+    for (Toolchain *toolchain : toolchains)
+        if (toolchain->typeId() == ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID)
+            clangToolchains.append(static_cast<GccToolchain *>(toolchain));
+    return clangToolchains;
 }
 
-static QList<ClangToolChain *> autoDetectedIosToolChains()
+static QList<GccToolchain *> autoDetectedIosToolchains()
 {
-    const QList<ClangToolChain *> toolChains = clangToolChains(ToolChainManager::toolChains());
-    return Utils::filtered(toolChains, [](ClangToolChain *toolChain) {
+    const QList<GccToolchain *> toolchains = clangToolchains(ToolchainManager::toolchains());
+    return filtered(toolchains, [](GccToolchain *toolChain) {
         return toolChain->isAutoDetected()
                && (toolChain->displayName().startsWith("iphone")
                    || toolChain->displayName().startsWith("Apple Clang")); // TODO tool chains should be marked directly
     });
 }
 
-static ToolChainPair findToolChainForPlatform(const XcodePlatform &platform,
+static ToolchainPair findToolchainForPlatform(const XcodePlatform &platform,
                                               const XcodePlatform::ToolchainTarget &target,
-                                              const QList<ClangToolChain *> &toolChains)
+                                              const QList<GccToolchain *> &toolchains)
 {
-    ToolChainPair platformToolChains;
-    auto toolchainMatch = [](ClangToolChain *toolChain, const Utils::FilePath &compilerPath, const QStringList &flags) {
+    ToolchainPair platformToolchains;
+    auto toolchainMatch = [](GccToolchain *toolChain, const FilePath &compilerPath, const QStringList &flags) {
         return compilerPath == toolChain->compilerCommand()
                 && flags == toolChain->platformCodeGenFlags()
                 && flags == toolChain->platformLinkerFlags();
     };
-    platformToolChains.first = Utils::findOrDefault(toolChains, std::bind(toolchainMatch, std::placeholders::_1,
+    platformToolchains.first = findOrDefault(toolchains, std::bind(toolchainMatch, std::placeholders::_1,
                                                                   platform.cCompilerPath,
                                                                   target.backendFlags));
-    platformToolChains.second = Utils::findOrDefault(toolChains, std::bind(toolchainMatch, std::placeholders::_1,
+    platformToolchains.second = findOrDefault(toolchains, std::bind(toolchainMatch, std::placeholders::_1,
                                                                    platform.cxxCompilerPath,
                                                                    target.backendFlags));
-    return platformToolChains;
+    return platformToolchains;
 }
 
-static QHash<XcodePlatform::ToolchainTarget, ToolChainPair> findToolChains(const QList<XcodePlatform> &platforms)
+static QHash<XcodePlatform::ToolchainTarget, ToolchainPair> findToolchains(const QList<XcodePlatform> &platforms)
 {
-    QHash<XcodePlatform::ToolchainTarget, ToolChainPair> platformToolChainHash;
-    const QList<ClangToolChain *> toolChains = autoDetectedIosToolChains();
+    QHash<XcodePlatform::ToolchainTarget, ToolchainPair> platformToolchainHash;
+    const QList<GccToolchain *> toolchains = autoDetectedIosToolchains();
     for (const XcodePlatform &platform : platforms) {
         for (const XcodePlatform::ToolchainTarget &target : platform.targets) {
-            ToolChainPair platformToolchains = findToolChainForPlatform(platform, target,
-                                                                        toolChains);
+            ToolchainPair platformToolchains = findToolchainForPlatform(platform, target,
+                                                                        toolchains);
             if (platformToolchains.first || platformToolchains.second)
-                platformToolChainHash.insert(target, platformToolchains);
+                platformToolchainHash.insert(target, platformToolchains);
         }
     }
-    return platformToolChainHash;
+    return platformToolchainHash;
 }
 
 static QSet<Kit *> existingAutoDetectedIosKits()
 {
-    return Utils::toSet(Utils::filtered(KitManager::kits(), [](Kit *kit) -> bool {
-        Core::Id deviceKind = DeviceTypeKitAspect::deviceTypeId(kit);
+    return toSet(filtered(KitManager::kits(), [](Kit *kit) -> bool {
+        Id deviceKind = DeviceTypeKitAspect::deviceTypeId(kit);
         return kit->isAutoDetected() && (deviceKind == Constants::IOS_DEVICE_TYPE
                                          || deviceKind == Constants::IOS_SIMULATOR_TYPE);
     }));
@@ -176,22 +162,22 @@ static QSet<Kit *> existingAutoDetectedIosKits()
 
 static void printKits(const QSet<Kit *> &kits)
 {
-    foreach (const Kit *kit, kits)
+    for (const Kit *kit : kits)
         qCDebug(kitSetupLog) << "  -" << kit->displayName();
 }
 
-static void setupKit(Kit *kit, Core::Id pDeviceType, const ToolChainPair& toolChains,
-                     const QVariant &debuggerId, const Utils::FilePath &sdkPath, BaseQtVersion *qtVersion)
+static void setupKit(Kit *kit, Id pDeviceType, const ToolchainPair& toolchains,
+                     const QVariant &debuggerId, const FilePath &sdkPath, QtVersion *qtVersion)
 {
     DeviceTypeKitAspect::setDeviceTypeId(kit, pDeviceType);
-    if (toolChains.first)
-        ToolChainKitAspect::setToolChain(kit, toolChains.first);
+    if (toolchains.first)
+        ToolchainKitAspect::setToolchain(kit, toolchains.first);
     else
-        ToolChainKitAspect::clearToolChain(kit, ProjectExplorer::Constants::C_LANGUAGE_ID);
-    if (toolChains.second)
-        ToolChainKitAspect::setToolChain(kit, toolChains.second);
+        ToolchainKitAspect::clearToolchain(kit, ProjectExplorer::Constants::C_LANGUAGE_ID);
+    if (toolchains.second)
+        ToolchainKitAspect::setToolchain(kit, toolchains.second);
     else
-        ToolChainKitAspect::clearToolChain(kit, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
+        ToolchainKitAspect::clearToolchain(kit, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
 
     QtKitAspect::setQtVersion(kit, qtVersion);
     // only replace debugger with the default one if we find an unusable one here
@@ -202,9 +188,8 @@ static void setupKit(Kit *kit, Core::Id pDeviceType, const ToolChainPair& toolCh
             && debuggerId.isValid())
         DebuggerKitAspect::setDebugger(kit, debuggerId);
 
-    kit->setMutable(DeviceKitAspect::id(), true);
     kit->setSticky(QtKitAspect::id(), true);
-    kit->setSticky(ToolChainKitAspect::id(), true);
+    kit->setSticky(ToolchainKitAspect::id(), true);
     kit->setSticky(DeviceTypeKitAspect::id(), true);
     kit->setSticky(SysRootKitAspect::id(), true);
     kit->setSticky(DebuggerKitAspect::id(), false);
@@ -212,7 +197,7 @@ static void setupKit(Kit *kit, Core::Id pDeviceType, const ToolChainPair& toolCh
     SysRootKitAspect::setSysRoot(kit, sdkPath);
 }
 
-static QVersionNumber findXcodeVersion(const Utils::FilePath &developerPath)
+static QVersionNumber findXcodeVersion(const FilePath &developerPath)
 {
     const FilePath xcodeInfo = developerPath.parentDir().pathAppended("Info.plist");
     if (xcodeInfo.exists()) {
@@ -229,14 +214,14 @@ static QByteArray decodeProvisioningProfile(const QString &path)
 {
     QTC_ASSERT(!path.isEmpty(), return QByteArray());
 
-    Utils::SynchronousProcess p;
-    p.setTimeoutS(3);
+    Process p;
     // path is assumed to be valid file path to .mobileprovision
-    const QStringList args = {"smime", "-inform", "der", "-verify", "-in", path};
-    Utils::SynchronousProcessResponse res = p.runBlocking("openssl", args);
-    if (res.result != Utils::SynchronousProcessResponse::Finished)
+    p.setCommand({"openssl", {"smime", "-inform", "der", "-verify", "-in", path}});
+    using namespace std::chrono_literals;
+    p.runBlocking(3s);
+    if (p.result() != ProcessResult::FinishedWithSuccess)
         qCDebug(iosCommonLog) << "Reading signed provisioning file failed" << path;
-    return res.stdOut().toLatin1();
+    return p.cleanedStdOut().toLatin1();
 }
 
 void IosConfigurations::updateAutomaticKitList()
@@ -247,9 +232,9 @@ void IosConfigurations::updateAutomaticKitList()
     qCDebug(kitSetupLog) << "Developer path:" << developerPath();
 
     // target -> tool chain
-    const auto targetToolChainHash = findToolChains(platforms);
+    const auto targetToolchainHash = findToolchains(platforms);
 
-    const auto qtVersions = Utils::toSet(QtVersionManager::versions([](const BaseQtVersion *v) {
+    const auto qtVersions = toSet(QtVersionManager::versions([](const QtVersion *v) {
         return v->isValid() && v->type() == Constants::IOSQT;
     }));
 
@@ -262,8 +247,8 @@ void IosConfigurations::updateAutomaticKitList()
     QSet<Kit *> resultingKits;
     for (const XcodePlatform &platform : platforms) {
         for (const auto &sdk : platform.sdks) {
-            const auto targets = Utils::filtered(platform.targets,
-                                           [&sdk](const XcodePlatform::ToolchainTarget &target) {
+            const auto targets = filtered(platform.targets,
+                                          [&sdk](const XcodePlatform::ToolchainTarget &target) {
                  return sdk.architectures.first() == target.architecture;
             });
             if (targets.empty())
@@ -271,25 +256,25 @@ void IosConfigurations::updateAutomaticKitList()
 
             const auto target = targets.front();
 
-            const ToolChainPair &platformToolchains = targetToolChainHash.value(target);
+            const ToolchainPair &platformToolchains = targetToolchainHash.value(target);
             if (!platformToolchains.first && !platformToolchains.second) {
                 qCDebug(kitSetupLog) << "  - No tool chain found";
                 continue;
             }
-            Core::Id pDeviceType = deviceId(sdk.directoryName);
+            Id pDeviceType = deviceId(sdk.directoryName);
             if (!pDeviceType.isValid()) {
                 qCDebug(kitSetupLog) << "Unsupported/Invalid device type" << sdk.directoryName;
                 continue;
             }
 
-            for (BaseQtVersion *qtVersion : qtVersions) {
+            for (QtVersion *qtVersion : qtVersions) {
                 qCDebug(kitSetupLog) << "  - Qt version:" << qtVersion->displayName();
-                Kit *kit = Utils::findOrDefault(existingKits, [&pDeviceType, &platformToolchains, &qtVersion](const Kit *kit) {
+                Kit *kit = findOrDefault(existingKits, [&pDeviceType, &platformToolchains, &qtVersion](const Kit *kit) {
                     // we do not compare the sdk (thus automatically upgrading it in place if a
                     // new Xcode is used). Change?
                     return DeviceTypeKitAspect::deviceTypeId(kit) == pDeviceType
-                            && ToolChainKitAspect::toolChain(kit, ProjectExplorer::Constants::CXX_LANGUAGE_ID) == platformToolchains.second
-                            && ToolChainKitAspect::toolChain(kit, ProjectExplorer::Constants::C_LANGUAGE_ID) == platformToolchains.first
+                            && ToolchainKitAspect::cxxToolchain(kit) == platformToolchains.second
+                            && ToolchainKitAspect::cToolchain(kit) == platformToolchains.first
                             && QtKitAspect::qtVersion(kit) == qtVersion;
                 });
                 QTC_ASSERT(!resultingKits.contains(kit), continue);
@@ -303,7 +288,7 @@ void IosConfigurations::updateAutomaticKitList()
                     const auto init = [&](Kit *k) {
                         k->setAutoDetected(true);
                         const QString baseDisplayName = isSimulatorDeviceId(pDeviceType)
-                                ? tr("%1 Simulator").arg(qtVersion->unexpandedDisplayName())
+                                ? Tr::tr("%1 Simulator").arg(qtVersion->unexpandedDisplayName())
                                 : qtVersion->unexpandedDisplayName();
                         k->setUnexpandedDisplayName(baseDisplayName);
                         setupKit(k, pDeviceType, platformToolchains, debuggerId, sdk.path, qtVersion);
@@ -318,8 +303,7 @@ void IosConfigurations::updateAutomaticKitList()
     existingKits.subtract(resultingKits);
     qCDebug(kitSetupLog) << "Removing unused kits:";
     printKits(existingKits);
-    foreach (Kit *kit, existingKits)
-        KitManager::deregisterKit(kit);
+    KitManager::deregisterKits(toList(existingKits));
 }
 
 static IosConfigurations *m_instance = nullptr;
@@ -357,19 +341,6 @@ void IosConfigurations::setIgnoreAllDevices(bool ignoreDevices)
     }
 }
 
-void IosConfigurations::setScreenshotDir(const FilePath &path)
-{
-    if (m_instance->m_screenshotDir != path) {
-        m_instance->m_screenshotDir = path;
-        m_instance->save();
-    }
-}
-
-FilePath IosConfigurations::screenshotDir()
-{
-    return m_instance->m_screenshotDir;
-}
-
 FilePath IosConfigurations::developerPath()
 {
     return m_instance->m_developerPath;
@@ -382,10 +353,9 @@ QVersionNumber IosConfigurations::xcodeVersion()
 
 void IosConfigurations::save()
 {
-    QSettings *settings = Core::ICore::settings();
+    QtcSettings *settings = Core::ICore::settings();
     settings->beginGroup(SettingsGroup);
-    settings->setValue(ignoreAllDevicesKey, m_ignoreAllDevices);
-    settings->setValue(screenshotDirPathKey, m_screenshotDir.toString());
+    settings->setValueWithDefault(ignoreAllDevicesKey, m_ignoreAllDevices, IgnoreAllDevicesDefault);
     settings->endGroup();
 }
 
@@ -399,16 +369,9 @@ IosConfigurations::IosConfigurations(QObject *parent)
 
 void IosConfigurations::load()
 {
-    QSettings *settings = Core::ICore::settings();
+    QtcSettings *settings = Core::ICore::settings();
     settings->beginGroup(SettingsGroup);
-    m_ignoreAllDevices = settings->value(ignoreAllDevicesKey, false).toBool();
-    m_screenshotDir = FilePath::fromString(settings->value(screenshotDirPathKey).toString());
-    if (!m_screenshotDir.exists()) {
-        QString defaultDir =
-                QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).constFirst();
-        m_screenshotDir = FilePath::fromString(defaultDir);
-    }
-
+    m_ignoreAllDevices = settings->value(ignoreAllDevicesKey, IgnoreAllDevicesDefault).toBool();
     settings->endGroup();
 }
 
@@ -416,13 +379,13 @@ void IosConfigurations::updateSimulators()
 {
     // currently we have just one simulator
     DeviceManager *devManager = DeviceManager::instance();
-    Core::Id devId = Constants::IOS_SIMULATOR_DEVICE_ID;
+    Id devId = Constants::IOS_SIMULATOR_DEVICE_ID;
     IDevice::ConstPtr dev = devManager->find(devId);
-    if (dev.isNull()) {
+    if (!dev) {
         dev = IDevice::ConstPtr(new IosSimulator(devId));
         devManager->addDevice(dev);
     }
-    SimulatorControl::updateAvailableSimulators();
+    Utils::futureSynchronizer()->addFuture(SimulatorControl::updateAvailableSimulators(this));
 }
 
 void IosConfigurations::setDeveloperPath(const FilePath &devPath)
@@ -474,22 +437,27 @@ void IosConfigurations::loadProvisioningData(bool notify)
     const QSettings xcodeSettings(xcodePlistPath, QSettings::NativeFormat);
     const QVariantMap teamMap = xcodeSettings.value(provisioningTeamsTag).toMap();
     QList<QVariantMap> teams;
-    QMapIterator<QString, QVariant> accountiterator(teamMap);
-    while (accountiterator.hasNext()) {
-        accountiterator.next();
-        QVariantMap teamInfo = accountiterator.value().toMap();
-        int provisioningTeamIsFree = teamInfo.value(freeTeamTag).toBool() ? 1 : 0;
-        teamInfo[freeTeamTag] = provisioningTeamIsFree;
-        teamInfo[emailTag] = accountiterator.key();
-        teams.append(teamInfo);
+    for (auto accountiterator = teamMap.cbegin(), end = teamMap.cend();
+            accountiterator != end; ++accountiterator) {
+        // difference between Qt 5 (map) and Qt 6 (list of maps)
+        const bool isList = accountiterator->userType() == QMetaType::QVariantList;
+        const QVariantList teamsList = isList ? accountiterator.value().toList()
+                                              : QVariantList({accountiterator.value()});
+        for (const QVariant &teamInfoIt : teamsList) {
+            QVariantMap teamInfo = teamInfoIt.toMap();
+            int provisioningTeamIsFree = teamInfo.value(freeTeamTag).toBool() ? 1 : 0;
+            teamInfo[freeTeamTag] = provisioningTeamIsFree;
+            teamInfo[emailTag] = accountiterator.key();
+            teams.append(teamInfo);
+        }
     }
 
     // Sort team id's to move the free provisioning teams at last of the list.
-    Utils::sort(teams, [](const QVariantMap &teamInfo1, const QVariantMap &teamInfo2) {
+    sort(teams, [](const QVariantMap &teamInfo1, const QVariantMap &teamInfo2) {
         return teamInfo1.value(freeTeamTag).toInt() < teamInfo2.value(freeTeamTag).toInt();
     });
 
-    foreach (auto teamInfo, teams) {
+    for (auto teamInfo : std::as_const(teams)) {
         auto team = std::make_shared<DevelopmentTeam>();
         team->m_name = teamInfo.value(teamNameTag).toString();
         team->m_email = teamInfo.value(emailTag).toString();
@@ -500,7 +468,10 @@ void IosConfigurations::loadProvisioningData(bool notify)
 
     const QDir provisioningProflesDir(provisioningProfileDirPath);
     const QStringList filters = {"*.mobileprovision"};
-    foreach (QFileInfo fileInfo, provisioningProflesDir.entryInfoList(filters, QDir::NoDotAndDotDot | QDir::Files)) {
+    const QList<QFileInfo> fileInfos = provisioningProflesDir.entryInfoList(filters,
+                                                                            QDir::NoDotAndDotDot
+                                                                                | QDir::Files);
+    for (const QFileInfo &fileInfo : fileInfos) {
         QDomDocument provisioningDoc;
         auto profile = std::make_shared<ProvisioningProfile>();
         QString teamID;
@@ -557,7 +528,7 @@ DevelopmentTeamPtr IosConfigurations::developmentTeam(const QString &teamID)
     QTC_CHECK(m_instance);
     m_instance->initializeProvisioningData();
     return findOrDefault(m_instance->m_developerTeams,
-                         Utils::equal(&DevelopmentTeam::identifier, teamID));
+                         equal(&DevelopmentTeam::identifier, teamID));
 }
 
 const ProvisioningProfiles &IosConfigurations::provisioningProfiles()
@@ -571,62 +542,68 @@ ProvisioningProfilePtr IosConfigurations::provisioningProfile(const QString &pro
 {
     QTC_CHECK(m_instance);
     m_instance->initializeProvisioningData();
-    return Utils::findOrDefault(m_instance->m_provisioningProfiles,
-                                Utils::equal(&ProvisioningProfile::identifier, profileID));
+    return findOrDefault(m_instance->m_provisioningProfiles,
+                                equal(&ProvisioningProfile::identifier, profileID));
 }
 
-static ClangToolChain *createToolChain(const XcodePlatform &platform,
-                                       const XcodePlatform::ToolchainTarget &target,
-                                       Core::Id l)
+class IosToolchainFactory final : public ToolchainFactory
 {
-    if (!l.isValid())
-        return nullptr;
+public:
+    IosToolchainFactory()
+    {
+        setSupportedLanguages({ProjectExplorer::Constants::C_LANGUAGE_ID,
+                               ProjectExplorer::Constants::CXX_LANGUAGE_ID});
+    }
 
-    if (l != Core::Id(ProjectExplorer::Constants::C_LANGUAGE_ID)
-            && l != Core::Id(ProjectExplorer::Constants::CXX_LANGUAGE_ID))
-        return nullptr;
+    Toolchains autoDetect(const ToolchainDetector &detector) const final;
 
-    auto toolChain = new ClangToolChain;
-    toolChain->setDetection(ToolChain::AutoDetection);
-    toolChain->setLanguage(l);
-    toolChain->setDisplayName(target.name);
-    toolChain->setPlatformCodeGenFlags(target.backendFlags);
-    toolChain->setPlatformLinkerFlags(target.backendFlags);
-    toolChain->resetToolChain(l == Core::Id(ProjectExplorer::Constants::CXX_LANGUAGE_ID) ?
-                                  platform.cxxCompilerPath : platform.cCompilerPath);
+    std::unique_ptr<ToolchainConfigWidget> createConfigurationWidget(
+        const ToolchainBundle &bundle) const override
+    {
+        return GccToolchain::createConfigurationWidget(bundle);
+    }
+};
 
-    return toolChain;
-}
-
-IosToolChainFactory::IosToolChainFactory()
+Toolchains IosToolchainFactory::autoDetect(const ToolchainDetector &detector) const
 {
-    setSupportedLanguages({ProjectExplorer::Constants::C_LANGUAGE_ID,
-                           ProjectExplorer::Constants::CXX_LANGUAGE_ID});
-}
+    if (detector.device->type() != ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE)
+        return {};
 
-QList<ToolChain *> IosToolChainFactory::autoDetect(const QList<ToolChain *> &existingToolChains)
-{
-    QList<ClangToolChain *> existingClangToolChains = clangToolChains(existingToolChains);
+    QList<GccToolchain *> existingClangToolchains = clangToolchains(detector.alreadyKnown);
     const QList<XcodePlatform> platforms = XcodeProbe::detectPlatforms().values();
-    QList<ToolChain *> toolChains;
-    toolChains.reserve(platforms.size());
-    foreach (const XcodePlatform &platform, platforms) {
+    Toolchains toolchains;
+    toolchains.reserve(platforms.size());
+    for (const XcodePlatform &platform : platforms) {
         for (const XcodePlatform::ToolchainTarget &target : platform.targets) {
-            ToolChainPair platformToolchains = findToolChainForPlatform(platform, target,
-                                                                        existingClangToolChains);
-            auto createOrAdd = [&](ClangToolChain* toolChain, Core::Id l) {
+            ToolchainPair platformToolchains = findToolchainForPlatform(platform, target,
+                                                                        existingClangToolchains);
+            auto createOrAdd = [&](GccToolchain *toolChain, Id l) {
                 if (!toolChain) {
-                    toolChain = createToolChain(platform, target, l);
-                    existingClangToolChains.append(toolChain);
+                    toolChain = new GccToolchain(ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID,
+                                                 GccToolchain::Clang);
+                    toolChain->setPriority(Toolchain::PriorityHigh);
+                    toolChain->setDetection(Toolchain::AutoDetection);
+                    toolChain->setLanguage(l);
+                    toolChain->setDisplayName(target.name);
+                    toolChain->setPlatformCodeGenFlags(target.backendFlags);
+                    toolChain->setPlatformLinkerFlags(target.backendFlags);
+                    toolChain->resetToolchain(l == ProjectExplorer::Constants::CXX_LANGUAGE_ID ?
+                                                  platform.cxxCompilerPath : platform.cCompilerPath);
+                    existingClangToolchains.append(toolChain);
                 }
-                toolChains.append(toolChain);
+                toolchains.append(toolChain);
             };
 
             createOrAdd(platformToolchains.first, ProjectExplorer::Constants::C_LANGUAGE_ID);
             createOrAdd(platformToolchains.second, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
         }
     }
-    return toolChains;
+    return toolchains;
+}
+
+void setupIosToolchain()
+{
+    static IosToolchainFactory theIosToolchainFactory;
 }
 
 QString DevelopmentTeam::identifier() const
@@ -641,15 +618,15 @@ QString DevelopmentTeam::displayName() const
 
 QString DevelopmentTeam::details() const
 {
-    return tr("%1 - Free Provisioning Team : %2")
-            .arg(m_identifier).arg(m_freeTeam ? tr("Yes") : tr("No"));
+    return Tr::tr("%1 - Free Provisioning Team : %2")
+            .arg(m_identifier).arg(m_freeTeam ? Tr::tr("Yes") : Tr::tr("No"));
 }
 
 QDebug &operator<<(QDebug &stream, DevelopmentTeamPtr team)
 {
     QTC_ASSERT(team, return stream);
     stream << team->displayName() << team->identifier() << team->isFreeProfile();
-    foreach (auto profile, team->m_profiles)
+    for (const auto &profile : std::as_const(team->m_profiles))
         stream << "Profile:" << profile;
     return stream;
 }
@@ -666,8 +643,8 @@ QString ProvisioningProfile::displayName() const
 
 QString ProvisioningProfile::details() const
 {
-    return tr("Team: %1\nApp ID: %2\nExpiration date: %3").arg(m_team->identifier()).arg(m_appID)
-            .arg(m_expirationDate.toLocalTime().toString(Qt::SystemLocaleShortDate));
+    return Tr::tr("Team: %1\nApp ID: %2\nExpiration date: %3").arg(m_team->identifier()).arg(m_appID)
+            .arg(QLocale::system().toString(m_expirationDate.toLocalTime(), QLocale::ShortFormat));
 }
 
 QDebug &operator<<(QDebug &stream, std::shared_ptr<ProvisioningProfile> profile)
