@@ -16,41 +16,44 @@
 
 #include <aggregation/aggregate.h>
 
+#include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/editormanager/ieditorfactory.h>
 #include <coreplugin/find/basetextfind.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/editormanager/editormanager.h>
 
 #include <extensionsystem/invoker.h>
 #include <extensionsystem/pluginmanager.h>
 
+#include <projectexplorer/project.h>
+#include <projectexplorer/projectmanager.h>
+
+#include <texteditor/fontsettings.h>
+#include <texteditor/texteditorsettings.h>
+
 #include <utils/algorithm.h>
 #include <utils/checkablemessagebox.h>
 #include <utils/completingtextedit.h>
 #include <utils/fileutils.h>
 #include <utils/icon.h>
-#include <utils/process.h>
+#include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
 #include <utils/temporarydirectory.h>
 #include <utils/theme/theme.h>
 
-#include <texteditor/fontsettings.h>
-#include <texteditor/texteditorsettings.h>
-
-#include <projectexplorer/project.h>
-#include <projectexplorer/projectmanager.h>
-
+#include <QAction>
+#include <QApplication>
+#include <QCompleter>
 #include <QDir>
 #include <QFileInfo>
+#include <QMessageBox>
 #include <QPointer>
 #include <QProcess>
+#include <QPushButton>
 #include <QSet>
 #include <QStringListModel>
 #include <QStyle>
 #include <QToolBar>
-#include <QAction>
-#include <QApplication>
-#include <QMessageBox>
-#include <QCompleter>
 
 #include <cstring>
 
@@ -100,6 +103,7 @@ static bool acceptsWordForCompletion(const QString &word)
 
 namespace VcsBase {
 
+using namespace Core;
 using namespace Internal;
 using namespace Utils;
 
@@ -143,10 +147,10 @@ void VcsBaseSubmitEditor::setParameters(const VcsBaseSubmitEditorParameters &par
 {
     d->m_parameters = parameters;
     d->m_file.setId(parameters.id);
-    d->m_file.setMimeType(QLatin1String(parameters.mimeType));
+    d->m_file.setMimeType(parameters.mimeType);
 
     setWidget(d->m_widget);
-    document()->setPreferredDisplayName(Tr::tr(d->m_parameters.displayName));
+    document()->setPreferredDisplayName(d->m_parameters.displayName);
 
     // Message font according to settings
     CompletingTextEdit *descriptionEdit = d->m_widget->descriptionEdit();
@@ -174,7 +178,7 @@ void VcsBaseSubmitEditor::setParameters(const VcsBaseSubmitEditorParameters &par
     const CommonVcsSettings &settings = commonSettings();
     // Add additional context menu settings
     if (!settings.submitMessageCheckScript().isEmpty()
-            || !settings.nickNameMailMap.value().isEmpty()) {
+            || !settings.nickNameMailMap().isEmpty()) {
         auto sep = new QAction(this);
         sep->setSeparator(true);
         d->m_widget->addDescriptionEditContextMenuAction(sep);
@@ -186,34 +190,30 @@ void VcsBaseSubmitEditor::setParameters(const VcsBaseSubmitEditorParameters &par
             d->m_widget->addDescriptionEditContextMenuAction(checkAction);
         }
         // Insert nick
-        if (!settings.nickNameMailMap.value().isEmpty()) {
+        if (!settings.nickNameMailMap().isEmpty()) {
             auto insertAction = new QAction(Tr::tr("Insert Name..."), this);
             connect(insertAction, &QAction::triggered, this, &VcsBaseSubmitEditor::slotInsertNickName);
             d->m_widget->addDescriptionEditContextMenuAction(insertAction);
         }
     }
     // Do we have user fields?
-    if (!settings.nickNameFieldListFile.value().isEmpty())
-        createUserFields(settings.nickNameFieldListFile.value());
+    if (!settings.nickNameFieldListFile().isEmpty())
+        createUserFields(settings.nickNameFieldListFile());
 
     // wrapping. etc
     slotUpdateEditorSettings();
     connect(&settings, &CommonVcsSettings::changed,
             this, &VcsBaseSubmitEditor::slotUpdateEditorSettings);
-    connect(Core::EditorManager::instance(), &Core::EditorManager::currentEditorChanged,
-            this, [this] {
-                if (Core::EditorManager::currentEditor() == this)
-                    updateFileModel();
-            });
-    connect(qApp, &QApplication::applicationStateChanged,
-            this, [this](Qt::ApplicationState state) {
-                if (state == Qt::ApplicationActive)
-                    updateFileModel();
-            });
+    connect(EditorManager::instance(), &EditorManager::currentEditorChanged, this, [this] {
+        if (EditorManager::currentEditor() == this)
+            updateFileModel();
+    });
+    connect(qApp, &QApplication::applicationStateChanged, this, [this](Qt::ApplicationState state) {
+        if (state == Qt::ApplicationActive)
+            updateFileModel();
+    });
 
-    auto aggregate = new Aggregation::Aggregate;
-    aggregate->add(new Core::BaseTextFind(descriptionEdit));
-    aggregate->add(this);
+    Aggregation::aggregate({this, new BaseTextFind(descriptionEdit)});
 }
 
 VcsBaseSubmitEditor::~VcsBaseSubmitEditor()
@@ -241,14 +241,12 @@ static inline QStringList fieldTexts(const QString &fileContents)
     return rc;
 }
 
-void VcsBaseSubmitEditor::createUserFields(const QString &fieldConfigFile)
+void VcsBaseSubmitEditor::createUserFields(const FilePath &fieldConfigFile)
 {
     FileReader reader;
-    if (!reader.fetch(FilePath::fromString(fieldConfigFile),
-                      QIODevice::Text,
-                      Core::ICore::dialogParent())) {
+    if (!reader.fetch(fieldConfigFile, QIODevice::Text, ICore::dialogParent()))
         return;
-    }
+
     // Parse into fields
     const QStringList fields = fieldTexts(QString::fromUtf8(reader.data()));
     if (fields.empty())
@@ -315,7 +313,7 @@ void VcsBaseSubmitEditor::setLineWrapWidth(int w)
     d->m_widget->setLineWrapWidth(w);
 }
 
-Core::IDocument *VcsBaseSubmitEditor::document() const
+IDocument *VcsBaseSubmitEditor::document() const
 {
     return &d->m_file;
 }
@@ -387,7 +385,7 @@ SubmitFileModel *VcsBaseSubmitEditor::fileModel() const
 QStringList VcsBaseSubmitEditor::rowsToFiles(const QList<int> &rows) const
 {
     if (rows.empty())
-        return QStringList();
+        return {};
 
     QStringList rc;
     const SubmitFileModel *model = fileModel();
@@ -438,11 +436,11 @@ void VcsBaseSubmitEditor::setDescriptionMandatory(bool v)
 
 enum { checkDialogMinimumWidth = 500 };
 
-void VcsBaseSubmitEditor::accept(VcsBasePluginPrivate *plugin)
+void VcsBaseSubmitEditor::accept(VersionControlBase *plugin)
 {
     auto submitWidget = static_cast<SubmitEditorWidget *>(this->widget());
 
-    Core::EditorManager::activateEditor(this, Core::EditorManager::IgnoreNavigationHistory);
+    EditorManager::activateEditor(this, EditorManager::IgnoreNavigationHistory);
 
     QString errorMessage;
     const bool canCommit = checkSubmitMessage(&errorMessage) && submitWidget->canSubmit(&errorMessage);
@@ -456,31 +454,30 @@ void VcsBaseSubmitEditor::accept(VcsBasePluginPrivate *plugin)
 void VcsBaseSubmitEditor::close()
 {
     d->m_disablePrompt = true;
-    Core::EditorManager::closeDocuments({document()});
+    EditorManager::closeDocuments({document()});
 }
 
-bool VcsBaseSubmitEditor::promptSubmit(VcsBasePluginPrivate *plugin)
+bool VcsBaseSubmitEditor::promptSubmit(VersionControlBase *plugin)
 {
     if (d->m_disablePrompt)
         return true;
 
-    Core::EditorManager::activateEditor(this, Core::EditorManager::IgnoreNavigationHistory);
+    EditorManager::activateEditor(this, EditorManager::IgnoreNavigationHistory);
 
     auto submitWidget = static_cast<SubmitEditorWidget *>(this->widget());
     if (!submitWidget->isEnabled() || !submitWidget->isEdited())
         return true;
 
-    QMessageBox mb(Core::ICore::dialogParent());
+    QMessageBox mb(ICore::dialogParent());
     mb.setWindowTitle(plugin->commitAbortTitle());
     mb.setIcon(QMessageBox::Warning);
     mb.setText(plugin->commitAbortMessage());
-    mb.setStandardButtons(QMessageBox::Close | QMessageBox::Cancel);
-    // On Windows there is no mnemonic for Close. Set it explicitly.
-    mb.button(QMessageBox::Close)->setText(Tr::tr("&Close"));
-    mb.button(QMessageBox::Cancel)->setText(Tr::tr("&Keep Editing"));
-    mb.setDefaultButton(QMessageBox::Cancel);
+    QPushButton *closeButton = mb.addButton(Tr::tr("&Close"), QMessageBox::AcceptRole);
+    QPushButton *keepButton = mb.addButton(Tr::tr("&Keep Editing"), QMessageBox::RejectRole);
+    mb.setDefaultButton(keepButton);
+    mb.setEscapeButton(keepButton);
     mb.exec();
-    return mb.result() == QMessageBox::Close;
+    return mb.clickedButton() == closeButton;
 }
 
 QString VcsBaseSubmitEditor::promptForNickName()
@@ -489,7 +486,7 @@ QString VcsBaseSubmitEditor::promptForNickName()
         d->m_nickNameDialog = new NickNameDialog(VcsPlugin::instance()->nickNameModel(), d->m_widget);
     if (d->m_nickNameDialog->exec() == QDialog::Accepted)
        return d->m_nickNameDialog->nickName();
-    return QString();
+    return {};
 }
 
 void VcsBaseSubmitEditor::slotInsertNickName()
@@ -521,7 +518,7 @@ void VcsBaseSubmitEditor::slotCheckSubmitMessage()
 
 bool VcsBaseSubmitEditor::checkSubmitMessage(QString *errorMessage) const
 {
-    const QString checkScript = commonSettings().submitMessageCheckScript.value();
+    const FilePath checkScript = commonSettings().submitMessageCheckScript();
     if (checkScript.isEmpty())
         return true;
     QApplication::setOverrideCursor(Qt::WaitCursor);
@@ -530,17 +527,17 @@ bool VcsBaseSubmitEditor::checkSubmitMessage(QString *errorMessage) const
     return rc;
 }
 
-static QString msgCheckScript(const FilePath &workingDir, const QString &cmd)
+static QString msgCheckScript(const FilePath &workingDir, const FilePath &cmd)
 {
-    const QString nativeCmd = QDir::toNativeSeparators(cmd);
+    const QString nativeCmd = cmd.toUserOutput();
     return workingDir.isEmpty() ?
            Tr::tr("Executing %1").arg(nativeCmd) :
-           Tr::tr("Executing [%1] %2").
-           arg(workingDir.toUserOutput(), nativeCmd);
+           Tr::tr("Executing [%1] %2").arg(workingDir.toUserOutput(), nativeCmd);
 }
 
-bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const QString &checkScript, QString *errorMessage) const
+bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const FilePath &checkScript, QString *errorMessage) const
 {
+    QTC_ASSERT(!checkScript.needsDevice(), return false); // Not supported below.
     // Write out message
     TempFileSaver saver(TemporaryDirectory::masterDirectoryPath() + "/msgXXXXXX.txt");
     saver.write(fileContents());
@@ -552,7 +549,7 @@ bool VcsBaseSubmitEditor::runSubmitMessageCheckScript(const QString &checkScript
     Process checkProcess;
     if (!d->m_checkScriptWorkingDirectory.isEmpty())
         checkProcess.setWorkingDirectory(d->m_checkScriptWorkingDirectory);
-    checkProcess.setCommand({FilePath::fromString(checkScript), {saver.filePath().toString()}});
+    checkProcess.setCommand({checkScript, {saver.filePath().path()}});
     checkProcess.start();
     const bool succeeded = checkProcess.waitForFinished();
 
@@ -601,4 +598,65 @@ void VcsBaseSubmitEditor::filterUntrackedFilesOfProject(const FilePath &reposito
     }
 }
 
+// Factories
+
+const char SUBMIT[] = "Vcs.Submit";
+const char DIFF_SELECTED[] = "Vcs.DiffSelectedFiles";
+
+class VcsSubmitEditorFactory final : public IEditorFactory
+{
+public:
+    VcsSubmitEditorFactory(VersionControlBase *versionControl,
+                           const VcsBaseSubmitEditorParameters &parameters)
+    {
+        QAction *submitAction = nullptr;
+        QAction *diffAction = nullptr;
+        QAction *undoAction = nullptr;
+        QAction *redoAction = nullptr;
+
+        const Context context(parameters.id);
+
+        ActionBuilder(versionControl, Core::Constants::UNDO)
+            .setText(Tr::tr("&Undo"))
+            .setContext(context)
+            .bindContextAction(&undoAction);
+
+        ActionBuilder(versionControl, Core::Constants::REDO)
+            .setText(Tr::tr("&Redo"))
+            .setContext(context)
+            .bindContextAction(&redoAction);
+
+        ActionBuilder(versionControl, SUBMIT)
+            .setText(versionControl->commitDisplayName())
+            .setIcon(VcsBaseSubmitEditor::submitIcon())
+            .setContext(context)
+            .bindContextAction(&submitAction)
+            .setCommandAttribute(Command::CA_UpdateText)
+            .addOnTriggered(versionControl, &VersionControlBase::commitFromEditor);
+
+        ActionBuilder(versionControl, DIFF_SELECTED)
+            .setText(Tr::tr("Diff &Selected Files"))
+            .setIcon(VcsBaseSubmitEditor::diffIcon())
+            .setContext(context)
+            .bindContextAction(&diffAction);
+
+        setId(parameters.id);
+        setDisplayName(parameters.displayName);
+        addMimeType(parameters.mimeType);
+        setEditorCreator([parameters, submitAction, diffAction, undoAction, redoAction] {
+            VcsBaseSubmitEditor *editor = parameters.editorCreator();
+            editor->setParameters(parameters);
+            editor->registerActions(undoAction, redoAction, submitAction, diffAction);
+            return editor;
+        });
+    }
+};
+
+void setupVcsSubmitEditor(VersionControlBase *versionControl,
+                          const VcsBaseSubmitEditorParameters &parameters)
+{
+    auto factory = new VcsSubmitEditorFactory(versionControl, parameters);
+    QObject::connect(versionControl, &QObject::destroyed, [factory] { delete factory; });
+}
+
 } // namespace VcsBase
